/* 

   This code was ported to the Sega Dreamcast ( all rights reserved )
   using KOS ( http://dcdev.allusion.net/ ) and its addon library KGL.
   
   It was ported by Stephen Bolton ( aka HyperG - hyper_gamer13@hotmail.com ).
   
   The BMP source code included with this port was coded by Paul Boese  ( http://www.axlen.com/ ).
   
   I also would like to thank Flecko for helping me get started with OpenGL.
   
*/


//
// This code was originally created by Jeff Molofee '99 
// (ported to Linux/GLUT by Richard Campbell '99)
//
// If you've found this code useful, please let me know.
//
// Visit me at www.demonews.com/hosted/nehe 
// (email Richard Campbell at ulmont@bellsouth.net)
//
#include <kos.h>
#include <GL\gl.h>			// Header File For The OpenGL32 Library
#include <GL\glu.h>			// Header File For The GLu32 Library
#include "bmp.h"


/* floats for x rotation, y rotation, z rotation */
float xrot, yrot, zrot;

//-------Start of DC Image Stuff------//

/* Load a texture using pcx_load_texture and glKosTex2D */
void loadtxr(const char *fn, GLuint *txr, GLint texformat, int alpha) {
	uint32 txaddr;
	int w, h;
	if (pcx_load_texture(fn, 1, alpha, &txaddr, &w, &h) < 0) {
		printf("can't load %s\n", fn);
		return;
	}

	glGenTextures(1, txr);
	glBindTexture(GL_TEXTURE_2D, *txr);
	glKosTex2D(texformat, w, h, txaddr);
}

// Loads a BMP file to a texture  - Use with transparent polys!
// BMP file should be at least 256x256.
// Black (R=G=B=0) is transparent.
void loadbmptxr(const char *fn, GLuint *txr) {
	uint32 txaddr;
	int w, h;

	BITMAP *myBmp;

	myBmp=load_bmp(fn);
	if (NULL != myBmp) {
		w = myBmp->nWidth;
		h = myBmp->nHeight;
		txaddr = ta_txr_allocate(w * h *2);
		ta_txr_load(txaddr, myBmp->data, w * h *2); 
		//txr_to_alpha(myBmp->data, w, h);
		//txr_twiddle_copy(myBmp->data, w, h, txaddr, w, h, 0);
		free(myBmp);
		printf("%s %dx%d\n", fn, w, h);
	} else {
		printf("*** %s didn't load ***\n", fn);
		return;
	}
	
	glGenTextures(1, txr);
	glBindTexture(GL_TEXTURE_2D, *txr);
	glKosTex2D(GL_ARGB1555, w, h, txaddr);
}

//--------End of DC Image stuff-------//

/* The function called when our window is resized (which shouldn't happen, because we're fullscreen) */
GLvoid ReSizeGLScene()
{
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    gluPerspective(45.0f,640.0f/480.0f,0.1f,100.0f);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void cube(GLuint texture) {
        glEnable(GL_TEXTURE_2D);
	glEnable(GL_KOS_AUTO_UV);
	glBindTexture( GL_TEXTURE_2D, texture);

	glLoadIdentity();				// Reset The View

    glTranslatef(0.0f,0.0f,-5.0f);              // move 5 units into the screen.
    
    glRotatef(xrot,1.0f,0.0f,0.0f);		// Rotate On The X Axis
    glRotatef(yrot,0.0f,1.0f,0.0f);		// Rotate On The Y Axis
    glRotatef(zrot,0.0f,0.0f,1.0f);		// Rotate On The Z Axis

    glBegin(GL_QUADS);		                // begin drawing a cube
    // Front Face (note that the texture's corners have to match the quad's corners)
    glVertex3f(-1.0f, 1.0f, 1.0f);	// Bottom Left Of The Texture and Quad
    glVertex3f(1.0f, 1.0f, 1.0f);	// Bottom Right Of The Texture and Quad
    glVertex3f(1.0f, -1.0f, 1.0f);	// Top Right Of The Texture and Quad
    glVertex3f(-1.0f, -1.0f, 1.0f);	// Top Left Of The Texture and Quad
    // Back Face
    glVertex3f(-1.0f, -1.0f, -1.0f);	// Bottom Right Of The Texture and Quad
    glVertex3f(1.0f, -1.0f, -1.0f);	// Top Right Of The Texture and Quad
    glVertex3f(1.0f, 1.0f, -1.0f);	// Top Left Of The Texture and Quad
    glVertex3f(-1.0f, 1.0f, -1.0f);	// Bottom Left Of The Texture and Quad
    // Left Face
    glVertex3f(-1.0f, 1.0f, 1.0f);	// Bottom Left Of The Texture and Quad
    glVertex3f(-1.0f, -1.0f, 1.0f);	// Bottom Right Of The Texture and Quad
    glVertex3f(-1.0f, -1.0f, -1.0f);	// Top Right Of The Texture and Quad
    glVertex3f(-1.0f,  1.0f, -1.0f);	// Top Left Of The Texture and Quad
    // Right face
    glVertex3f(1.0f, 1.0f, -1.0f);	// Bottom Right Of The Texture and Quad
    glVertex3f(1.0f, -1.0f, -1.0f);	// Top Right Of The Texture and Quad
    glVertex3f(1.0f, -1.0f, 1.0f);	// Top Left Of The Texture and Quad
    glVertex3f(1.0f, 1.0f, 1.0f);	// Bottom Left Of The Texture and Quad
    // Top Face
    glVertex3f(1.0f, 1.0f, 1.0f);	// Top Left Of The Texture and Quad
    glVertex3f(-1.0f, 1.0f, 1.0f);	// Bottom Left Of The Texture and Quad
    glVertex3f(-1.0f, 1.0f, -1.0f);	// Bottom Right Of The Texture and Quad
    glVertex3f(1.0f, 1.0f, -1.0f);	// Top Right Of The Texture and Quad
    // Bottom Face       
    glVertex3f(1.0f, -1.0f, -1.0f);	// Top Right Of The Texture and Quad
    glVertex3f(-1.0f, -1.0f, -1.0f);	// Top Left Of The Texture and Quad
    glVertex3f(-1.0f, -1.0f, 1.0f);	// Bottom Left Of The Texture and Quad
    glVertex3f(1.0f, -1.0f, 1.0f);	// Bottom Right Of The Texture and Quad
    
    glEnd();                                    // done with the polygon.

    xrot+=0.1f;//default 15.0f --PC            // X Axis Rotation	
    yrot+=0.1f; 		               // Y Axis Rotation
    zrot+=0.1f;		                       // Z Axis Rotation
    glDisable(GL_KOS_AUTO_UV);
    glDisable(GL_TEXTURE_2D);
}

extern uint8 romdisk[];
int main(int argc, char **argv) 
{
	uint8 c;
	cont_cond_t cond;
	GLuint tex_nehe;
	
	kos_init_all(THD_ENABLE | IRQ_ENABLE | TA_ENABLE, romdisk);	// set up KOS
        glKosInit();
        glEnable(GL_CULL_FACE);
    
        loadbmptxr("/rd/NeHe.bmp", &tex_nehe);
        c = maple_first_controller();
        glShadeModel(GL_SMOOTH);			// Enables Smooth Color Shading
    
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();				// Reset The Projection Matrix
    
        glMatrixMode(GL_MODELVIEW);
        
while(1) 
							// Infinite loop, the best kind			
	{
		/* Check key status */
		if (cont_get_cond(c, &cond) < 0)
			break;
		if (!(cond.buttons & CONT_START))
			break;

		
		glKosBeginFrame();			// Tell KOS to start the frame
		ReSizeGLScene();  			// This isn't really a needed function...but you get the idea
		cube(tex_nehe);				// Draw the scene
		glKosFinishFrame();			// Tell KOS you're done with the frame
        }
	
	return 0;
}